'''
    xam.repos
    ---------

    Contains URLs for the official XBMC addon repositories.

    Variable names surface as command line arguments to the --repo
    option. Each tuple contains the url to the addons.xml, the data dir
    url and the checksum url.

    :copyright: (c) 2012 Jonathan Beluch
    :license: BSD, see LICENSE for more details.
'''


FRODO= (
    'http://mirrors.xbmc.org/addons/frodo/addons.xml',
    'http://mirrors.xbmc.org/addons/frodo/',
    'http://mirrors.xbmc.org/addons/frodo/addons.xml.md5',
)

FRODO_PRE = (
    'http://mirrors.xbmc.org/addons/frodo-pre/addons.xml',
    'http://mirrors.xbmc.org/addons/frodo-pre/',
    'http://mirrors.xbmc.org/addons/frodo-pre/addons.xml.md5',
)

DHARMA = (
    'http://mirrors.xbmc.org/addons/dharma/addons.xml',
    'http://mirrors.xbmc.org/addons/dharma/',
    'http://mirrors.xbmc.org/addons/dharma/addons.xml.md5',
)

DHARMA_PRE = (
    'http://mirrors.xbmc.org/addons/dharma-pre/addons.xml',
    'http://mirrors.xbmc.org/addons/dharma-pre/',
    'http://mirrors.xbmc.org/addons/dharma-pre/addons.xml.md5',
)

EDEN = (
    'http://mirrors.xbmc.org/addons/eden/addons.xml',
    'http://mirrors.xbmc.org/addons/eden/',
    'http://mirrors.xbmc.org/addons/eden/addons.xml.md5',
)

EDEN_PRE = (
    'http://mirrors.xbmc.org/addons/eden-pre/addons.xml',
    'http://mirrors.xbmc.org/addons/eden-pre/',
    'http://mirrors.xbmc.org/addons/eden-pre/addons.xml.md5',
)
